@@ -1,3 +1,25 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/*
+   Copyright (C) 2000 CodeFactory AB
+   Copyright (C) 2000 Jonas Borgstr\366m <jonas@codefactory.se>
+   Copyright (C) 2000 Anders Carlsson <andersca@codefactory.se>
+
+   This library is free software; you can redistribute it and/or
+   modify it under the terms of the GNU Library General Public
+   License as published by the Free Software Foundation; either
+   version 2 of the License, or (at your option) any later version.
+
+   This library is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
+   Library General Public License for more details.
+
+   You should have received a copy of the GNU Library General Public License
+   along with this library; see the file COPYING.LIB.  If not, write to
+   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.
+*/
+
 #include <string.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -471,6 +493,7 @@ css_parse_color (CssValue *val, HtmlColor *color)
 		return FALSE;
 	
 	if (color) {
+		color->refcount = tmp_color->refcount;
 		color->red = tmp_color->red;
 		color->green = tmp_color->green;
 		color->blue = tmp_color->blue;
@@ -1270,7 +1293,7 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 		break;
 
 	case HTML_ATOM_TEXT_ALIGN: {
-		HtmlTextAlignType type = parent_style->inherited->text_align;
+		HtmlTextAlignType type = HTML_TEXT_ALIGN_DEFAULT;
 		switch (val->v.atom) {
 		case HTML_ATOM_INHERIT:
 			type = parent_style->inherited->text_align;
@@ -1287,6 +1310,9 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 		case HTML_ATOM_JUSTIFY:
 			type = HTML_TEXT_ALIGN_JUSTIFY;
 			break;
+		case HTML_ATOM_BOTTOM:
+			g_warning ("text-align:bottom not supported");
+			break;
 		default:
 			g_assert_not_reached ();
 			break;
@@ -2328,7 +2354,7 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 		break;
 		
 	default:
-		g_warning ("Unhandled property: %d %s\n", prop, html_atom_list_get_string (html_atom_list, prop));
+		g_print ("Unhandled property: %d %s\n", prop, html_atom_list_get_string (html_atom_list, prop));
 	}
 }
 
@@ -2422,7 +2448,10 @@ css_matcher_apply_stylesheet (HtmlDocument *doc, CssStylesheet *ss, xmlNode *nod
 					
 					entry->spec = sel->a * 1000000 + sel->b * 1000 + sel->c;
 					entry->type = type;
-					entry->decl = decl;
+					entry->decl = g_new (CssDeclaration, 1);
+					entry->decl->property = decl->property;
+					entry->decl->expr = css_value_ref (decl->expr);
+					entry->decl->important = decl->important;
 					*declaration_list = g_list_insert_sorted (*declaration_list, entry, css_declaration_list_sorter);
 				}
 			}
@@ -2449,6 +2478,7 @@ css_matcher_validate_style (HtmlStyle *style)
 static void
 free_decl_entry (CssDeclarationListEntry *entry, gpointer user_data)
 {
+	css_value_unref (entry->decl->expr);
 	g_free (entry->decl);
 	g_free (entry);
 }
@@ -2729,6 +2759,7 @@ css_matcher_get_style (HtmlDocument *doc, HtmlStyle *parent_style, xmlNode *node
 	HtmlFontSpecification *font_spec = parent_style ? parent_style->inherited->font_spec : NULL;
 	xmlChar *prop;
 	GList *declaration_list = NULL;
+	GList *temp;
 	GSList *ss;
 	HtmlStyle *style;
 
@@ -2768,17 +2799,20 @@ css_matcher_get_style (HtmlDocument *doc, HtmlStyle *parent_style, xmlNode *node
 				CssDeclaration *decl = rs->decl[i];
 
 				entry->type = CSS_STYLESHEET_STYLEDECL;
-				entry->decl = decl;
+				entry->decl = g_new (CssDeclaration, 1);
+				entry->decl->property = decl->property;
+				entry->decl->expr = css_value_ref (decl->expr);
+				entry->decl->important = decl->important;
 				entry->spec = 0;
 
 				declaration_list = g_list_insert_sorted (declaration_list, entry, css_declaration_list_sorter);
 			}
-			g_free (rs->decl);
-			g_free (rs);
+			css_ruleset_destroy (rs);
 		}
 		xmlFree (prop);
 	}
 
+	temp = declaration_list;
 	while (declaration_list) {
 		CssDeclarationListEntry *entry = declaration_list->data;
 
@@ -2786,6 +2820,7 @@ css_matcher_get_style (HtmlDocument *doc, HtmlStyle *parent_style, xmlNode *node
 
 		declaration_list = declaration_list->next;
 	}
+	declaration_list = temp;
 	g_list_foreach (declaration_list, (GFunc) free_decl_entry, NULL);
 	g_list_free (declaration_list);
 
